
package com.sun.jersey.samples.contextresolver;

import com.sun.grizzly.http.SelectorThread;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.container.grizzly.GrizzlyWebContainerFactory;
import java.io.IOException;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.ext.ContextResolver;
import javax.ws.rs.ext.Provider;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.annotation.XmlRootElement;


public class Main {

    @XmlRootElement
    public static class JAXBBean {

        public String value;

        public JAXBBean() {}

        public JAXBBean(String str) {
            value = str;
        }
    }

    @Produces("text/xml")
    @Provider
    public static class MyResolver implements ContextResolver<Marshaller> {

        JAXBContext c;

        public MyResolver() throws JAXBException {
            c = JAXBContext.newInstance(JAXBBean.class);
        }

        public Marshaller getContext(Class<?> type) {
            if (type == JAXBBean.class) {
                try {
                Marshaller m = c.createMarshaller();
                m.setProperty("jaxb.formatted.output", true);
                return m;
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            } else {
                return null;
            }
        }

    }

    @Path("/")
    public static class MyResource {
        @GET
        @Produces("text/xml")
        public JAXBBean get() {
            return new JAXBBean("some content");
        }
    }


    public static final URI BASE_URI = UriBuilder.fromUri("http://localhost/").port(9998).build();

    protected static SelectorThread startServer() throws IOException {
        final Map<String, String> initParams = new HashMap<String, String>();

        initParams.put("com.sun.jersey.config.property.packages", 
                "com.sun.jersey.samples.contextresolver");

        System.out.println("Starting grizzly...");
        SelectorThread threadSelector = GrizzlyWebContainerFactory.create(BASE_URI, initParams);     
        return threadSelector;
    }
    
    public static void main(String[] args) throws IOException {
        SelectorThread threadSelector = null;
        try {
            threadSelector = startServer();


            Client c = new Client();
            WebResource r = c.resource(BASE_URI);

            String s = r.get(String.class);
            System.out.println(s);
        } finally {
            threadSelector.stopEndpoint();
        }
    }    
}
